//========================================================================================
//
//	File:				Actions.h
//	Release Version:	$ ODF 1 $
//
//	Author:				Jim Lloyd
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef ACTIONS_H
#define ACTIONS_H

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__OSA__)
#include <OSA.h>
#endif

//========================================================================================
//	Forward declarations
//========================================================================================

class ODStorageUnit;

//========================================================================================
//	class CAction
//========================================================================================

class CAction
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:	
	CAction();
	virtual ~CAction();
	
	static FW_Boolean IsInStorage(Environment* ev, ODStorageUnit* storage);
	static CAction* CreateFromStorage(Environment* ev, ODStorageUnit* storage);
	static void WriteToStorage(Environment* ev, CAction* action, ODStorageUnit* storage);

//----------------------------------------------------------------------------------------
//	Action protocol
//
public:	
	virtual void Internalize(Environment* ev, 
						ODStorageUnit* storage) = 0;
						
	virtual void Externalize(Environment* ev, 
						ODStorageUnit* storage) = 0;
	
	virtual void DoIt() = 0;
};

//========================================================================================
//	class CSoundAction
//========================================================================================

class CSoundAction : public CAction
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CSoundAction(Environment* ev, ODStorageUnit* storage);
	CSoundAction(Environment* ev, FW_CReadableStream& stream);
	virtual ~CSoundAction();
	
	static FW_Boolean IsInStorage(Environment* ev, ODStorageUnit* storage);
	
//----------------------------------------------------------------------------------------
//	Archiving
//
public:
	static void* Read(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void Write(FW_CWritableStream& stream, FW_ClassTypeConstant type, const void *object);

//----------------------------------------------------------------------------------------
//	Action protocol
//
public:
	virtual void Internalize(Environment* ev, ODStorageUnit* storage);
	virtual void Externalize(Environment* ev, ODStorageUnit* storage);
	virtual void DoIt();

//----------------------------------------------------------------------------------------
//	Private implementation
//
private:
#if 0
	void InternalizeSound(Environment* ev, ODStorageUnit* storage);
#endif
	void InternalizeSoundFile(Environment* ev, ODStorageUnit* storage);
	void InternalizeFromStream(Environment* ev, FW_CReadableStream& stream);

	void ExternalizeToStream(Environment* ev, FW_CWritableStream& stream);

	Handle fSoundHandle;
};

//========================================================================================
//	class CScriptAction
//========================================================================================

class CScriptAction : public CAction
{
public:
	FW_DECLARE_CLASS

public:	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
	CScriptAction(Environment* ev, ODStorageUnit* storage);
	CScriptAction(Environment* ev, FW_CReadableStream& stream);
	virtual ~CScriptAction();
	
	static FW_Boolean IsInStorage(
						Environment* ev, 
						ODStorageUnit* storage);
	
//----------------------------------------------------------------------------------------
//	Archiving
//
public:
	static void*		Read(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void 		Write(FW_CWritableStream& stream, FW_ClassTypeConstant type, const void *object);

//----------------------------------------------------------------------------------------
//	Action protocol
//
public:
	virtual void Internalize(Environment* ev, 
						ODStorageUnit* storage);
	virtual void Externalize(Environment* ev, 
						ODStorageUnit* storage);
	
	virtual void DoIt();

//----------------------------------------------------------------------------------------
//	Private implementation
//
private:
#if 0
	void InternalizeScript(Environment* ev ODStorageUnit* storage);
#endif
	void InternalizeScriptFile(Environment* ev, ODStorageUnit* storageUnit);
	void InternalizeFromStream(Environment* ev, FW_CReadableStream& stream);

	void ExternalizeToStream(Environment* ev, FW_CWritableStream& stream);
	
	inline ComponentInstance GetScriptingComponent();
	inline void ReleaseScriptingComponent();

	void LoadAndExecuteScript(Handle scriptData);

	Handle fScriptHandle;
	
	static ComponentInstance fgComponentInstance;
};

#endif

//----------------------------------------------------------------------------------------
// CScriptAction::GetScriptingComponent
//----------------------------------------------------------------------------------------

inline ComponentInstance CScriptAction::GetScriptingComponent()
{
	if (!fgComponentInstance)
		fgComponentInstance = ::OpenDefaultComponent(kOSAComponentType, kOSAGenericScriptingComponentSubtype);
	
	return fgComponentInstance;
}

//----------------------------------------------------------------------------------------
// CScriptAction::ReleaseScriptingComponent
//----------------------------------------------------------------------------------------

inline void CScriptAction::ReleaseScriptingComponent()
{
	if (fgComponentInstance)
	{
		CloseComponent(fgComponentInstance);
		fgComponentInstance = NULL;
	}
}